(function() {
    const log       = require("ko/logging").getLogger("commando-scope-sections")
    const commando  = require("commando/commando");
    const {Cc, Ci}  = require("chrome");
    const prefs     = ko.prefs;
    
    const scope     = Cc["@activestate.com/commando/koScopeSections;1"].getService(Ci.koIScopeSections);
    const partSvc   = Cc["@activestate.com/koPartService;1"].getService(Ci.koIPartService);

    var local = {};

    //log.setLevel(require("ko/logging").LOG_DEBUG);

    this.onSearch = function(query, uuid, onComplete)
    {
        log.debug(uuid + " - Starting Scoped Search");

        if (!ko.codeintel.isActive) {
            log.debug("CodeIntel is not active");
            return onComplete();
        }
        
        var view = require("ko/views").current();
        
        var completed = 0;
        var searches = 1;
        
        var _onComplete = () =>
        {
            if (++completed == searches)
            {
                onComplete();
            }
        };
        
        if (view.get() && ("domviewer" in ko) &&
            ! ko.domviewer.viewMgr.isDisabledForCurrentView &&
            ko.domviewer.viewMgr.isSupportedDomLanguage(view.get()))
        {
            searches++;
            onSearchDOM(query, uuid, _onComplete, view);
        }
        
        onSearchCodeIntel(query, uuid, _onComplete, view);
    };
    
    var onSearchDOM = function(query, uuid, onComplete, view)
    {
        log.debug("onSearchDOM");
        
        if ( ! view.get("domView"))
        {
            log.debug("domView undefined, initializing dom viewer");
            ko.domviewer.viewMgr.getDOMView(view.get());
            view.get().domView.resetCb(function()
            {
                onSearchDOM(query, uuid, onComplete, view);
            });
            return;
        }
        
        var words = query.toLowerCase().split(" ");
        words = words.filter(function(w) !!w);
        
        var nodes;
        try
        {
            nodes = JSON.parse(view.get().domView.get_nodes_json());
        } catch (e)
        {
            onComplete();
            return;
        }
        
        var commands = [];
        for (let k of Object.keys(nodes))
        {
            let node = nodes[k];
            let name = node.tag;
            
            if ("attr_name" in node)
            {
                name += " ("+node.attr_name+"="+node.attr_val+")";
            }
            
            if (words.length)
            {
                var _name = name.toLowerCase();
                if ( ! words.every(function(w) _name.indexOf(w) != -1))
                {
                    continue;
                }
            }
            
            commands.push({
                id: "section-" + node.tag + node.line + node.col,
                name: name,
                description: "line " + node.line + ", col " + node.col,
                icon: "chrome://komodo/skin/images/codeintel/cb_xml_element.svg",
                scope: "scope-sections",
                allowMultiSelect: false,
                weight: 150,
                data: {
                    type: "DOM",
                    line: node.line,
                    col: node.col
                }
            });
        }
        
        if (commands.length)
            commando.renderResults(commands, uuid);
        
        onComplete();
    };
    
    var onSearchCodeIntel = function(query, uuid, onComplete, view = null)
    {
        log.debug("onSearchCodeIntel");
        
        var curProject = partSvc.currentProject;
        var cwd = curProject ? curProject.liveDirectory : ko.uriparse.URIToPath(ko.places.getDirectory());
        
        scope.onSearch(query, uuid, cwd, view.filePath, view.language, function(status, results) {
            commando.renderResults(JSON.parse(results), uuid);
        }.bind(this), onComplete);
    }
    
    this.onSelectResult = function(selectedItems)
    {
        log.debug("Navigating to section");
        
        var item = selectedItems.pop();
        
        var view = ko.views.manager.currentView;
        if (!view || view.getAttribute("type") != "editor") {
            log.debug("No editor view");
            return;
        }

        let data = item.resultData.data;
        
        if (data.type == "DOM")
        {
            view.scimoz.gotoLine(data.line-1); // 1-based -> 0-based
        }
        else
        {
            ko.open.URIAtLine(ko.uriparse.pathToURI(data.filepath), data.lineno);
        }

        commando.hideCommando();
    }
    
    var currentView = null;
    
    this.onShow = function()
    {
        var view = ko.views.manager.currentView;
        if (view == currentView) return;
        
        currentView = view;
        
        commando.search("");
    }

}).apply(module.exports);
